@@ -1,5 +1,5 @@
 """Build webapps using IPywidgets"""
-__version__ = "1.0.1"
+__version__ = "1.2.0"
 github_url = "https://github.com/widgetti/solara"
 git_branch = "master"
 
@@ -14,6 +14,7 @@ def _using_solara_server():
 from reacton import (
     component,
     component_interactive,
+    value_component,
     create_context,
     display,
     get_widget,
@@ -36,7 +37,8 @@ from . import util
 # flake8: noqa: F402
 from .datatypes import *
 from .hooks import *
-from .memoize import memoize
+from .cache import memoize
+from . import cache
 from .components import *
 
 from .routing import use_route, use_router, use_route_level, find_route, use_pathname, resolve_path
