@@ -7,6 +7,7 @@ from pathlib import Path
 from types import ModuleType
 from typing import Any, Callable, List, Optional, cast
 
+import ipywidgets
 import reacton
 import reacton.core
 
@@ -18,6 +19,9 @@ autoroute_level_context = solara.create_context(0)
 DEBUG = False
 
 
+DefaultLayout = solara.AppLayout
+
+
 def source_to_module(path: Path) -> ModuleType:
     fullname = path.stem
     mod = ModuleType(fullname)
@@ -62,6 +66,10 @@ def arg_cast(args: List[str], f: Callable):
             # raise TypeError(f"{f} expected positional argument {param}, but not enought argument supplied")
         annotation = param.annotation
         value = args[i]
+        check_optional_types = [str, int, float]
+        for check_type in check_optional_types:
+            if annotation == Optional[check_type]:
+                annotation = check_type
         if annotation == param.empty:
             results.append(value)
         else:
@@ -75,7 +83,7 @@ def arg_cast(args: List[str], f: Callable):
 @solara.component
 def RoutingProvider(children: List[reacton.core.Element] = [], routes: List[solara.Route] = [], pathname: str = ""):
     """Wraps the app, adds extra context, like navigation/routing."""
-    path, set_path = solara.use_state_or_update(pathname, key="solara-context-path")
+    path, set_path = solara.use_state(pathname, key="solara-context-path")
     nav = solara.Navigator(location=path, on_location=set_path)
     solara.routing._location_context.provide(solara.routing._Location(path, set_path))
     solara.routing.router_context.provide(solara.routing.Router(path, routes=routes, set_path=set_path))
@@ -95,6 +103,9 @@ def RenderPage():
     """Renders the page that matches the route."""
     level_start = solara.use_route_level()
     router = solara.use_context(solara.routing.router_context)
+    # we use these to cache script runs that use regular ipywidgets
+    modules = solara.use_memo(dict, dependencies=[])
+    modules_modified_times = solara.use_memo(dict, dependencies=[])
 
     if len(router.path_routes) <= level_start:
         with solara.VBox() as main:
@@ -182,17 +193,45 @@ def RenderPage():
         title_element = solara.Title(title)
         module = route_current.module
         namespace = module.__dict__
-        if "app" in namespace:
-            element = namespace["app"]
+        Page = namespace.get("Page", None)
+        # app is for backwards compatibility
+        page = namespace.get("page", namespace.get("app"))
+        if page is not None:
+            if isinstance(page, reacton.core.Element):
+                pass  # we are good
+            elif isinstance(page, ipywidgets.Widget):
+                # If we have a widget, we need to execute this again for each
+                # connection, since we cannot share widgets between connections/users.
+                # We also cannot tear them down, so we cache the widget based pages.
+                # To support hot reload, we manualy need to check the mtimes
+                # because the reload support for modules in reloader.py only works
+                # for modules.
+                if route_current.file is None:
+                    page = solara.Error(f"{route_current.path} is not associated with a file")
+                else:
+                    assert route_current.file is not None
+                    if route_current.path not in modules:
+                        modules[route_current.path] = source_to_module(route_current.file)
+                        modules_modified_times[route_current.path] = route_current.file.stat().st_mtime
+                    else:
+                        if modules_modified_times[route_current.path] != route_current.file.stat().st_mtime:
+                            # out of date, 'reload'
+                            modules[route_current.path] = source_to_module(route_current.file)
+                            modules_modified_times[route_current.path] = route_current.file.stat().st_mtime
+                    page = getattr(modules[route_current.path], "app", None)
+                    page = getattr(modules[route_current.path], "page", page)
+                    if page is None:
+                        page = solara.Error(f"{module} does not have a `Page` component of a `page` element or widget")
+            else:
+                page = solara.Error(f"{module} page variable not a Solara element or ipywidget")
             main = solara.Div(
                 children=[
                     title_element,
-                    element,
+                    page,
                 ]
             )
             main = wrap_in_layouts(main, layouts)
-        elif "Page" in namespace:
-            Page = get_page(module)
+        elif Page is not None:
             args = get_args(Page)
             main = solara.Div(
                 children=[
@@ -202,48 +241,11 @@ def RenderPage():
             )
             main = wrap_in_layouts(main, layouts)
         else:
-            with DefaultLayout(router_level=-1) as main:
+            with DefaultLayout() as main:
                 solara.Error(f"{module} does not have a Page component or an app element")
     return main
 
 
-@solara.component
-def DefaultLayout(children: List[reacton.core.Element] = [], router_level=-1):
-    """Default layout used in multipage applications.
-
-    ## See
-
-     * [Multipage](/docs/guides/multipage).
-     * [Understanding Routing](/docs/understanding/routing).
-
-    """
-    route_current, all_routes = solara.use_route()
-    router = solara.use_router()
-    selected = router.path
-
-    with solara.HBox(grow=True) as main:
-        with rv.NavigationDrawer(right=False, width="400px", v_model=True, permanent=True):
-            with rv.List(dense=True):
-                with rv.ListItemGroup(v_model=selected):
-                    for route in all_routes:
-                        if route.children and route.data is None:
-                            with solara.ListItem(route.label):
-                                for child in route.children:
-                                    path = solara.resolve_path(child)
-                                    with solara.Link(path):
-                                        title = child.label or "no label"
-                                        if callable(title):
-                                            title = "Error: dynamic title"
-                                        solara.ListItem(title, value=path)
-                        else:
-                            path = solara.resolve_path(route)
-                            with solara.Link(path):
-                                solara.ListItem(route.label, value=path)
-        with solara.Padding(4):
-            solara.Div(children=children)
-    return main
-
-
 def get_page(module: ModuleType, required=True):
     page = getattr(module, "Page", None)
     if required and page is None:
@@ -302,7 +304,7 @@ def generate_routes(module: ModuleType) -> List[solara.Route]:
     files are not Python modules.
 
 
-    See [our multipage guide](/docs/guides/multipage#as-a-package) for more details.
+    See [our multipage guide](/docs/howto/multipage#as-a-package) for more details.
 
 
     """
@@ -378,7 +380,7 @@ def generate_routes_directory(path: Path) -> List[solara.Route]:
     Python files ending in .py, or Jupyter notebooks ending in .ipynb
     will be executed and its `Page` component will be rendered.
 
-    Automatic titles will be [generated as explained in the multipage guide](/docs/guides/multipage).
+    Automatic titles will be [generated as explained in the multipage guide](/docs/howto/multipage).
 
     """
     from .server import reload
