@@ -13,6 +13,7 @@ from typing import Any, Callable, Dict, List, Optional, cast
 
 import ipywidgets as widgets
 import reacton
+from ipywidgets import DOMWidget, Widget
 from reacton.core import Element, render
 
 import solara
@@ -30,6 +31,8 @@ logger = logging.getLogger("solara.server.app")
 state_directory = Path(".") / "states"
 state_directory.mkdir(exist_ok=True)
 
+reload.reloader.start()
+
 
 class AppType(str, Enum):
     SCRIPT = "script"
@@ -42,6 +45,19 @@ def display(*args, **kwargs):
     print("display not implemented", args, kwargs)  # noqa
 
 
+_warned_is_widget = False
+
+
+def warn_is_widget():
+    global _warned_is_widget
+    if not _warned_is_widget:
+        _warned_is_widget = True
+        print(  # noqa
+            "The app you are running is a widget, note that your app is run once before any user connected. "
+            "If this causes a performance issue, please open an issue on GitHub."
+        )
+
+
 class AppScript:
     directory: Path
     routes: List[solara.Route]
@@ -61,14 +77,29 @@ class AppScript:
         else:
             self.name = name
         self.path: Path = Path(self.name)
-
-        app = self._execute()
+        try:
+            context = get_current_context()
+        except RuntimeError:
+            context = None
+        if context is not None:
+            raise RuntimeError(f"We should not have an existing Solara app context when running an app for the first time: {context}")
+        app_context = create_dummy_context()
+        with app_context:
+            app = self._execute()
         self._first_execute_app = app
         self._is_widget = isinstance(self._first_execute_app, widgets.Widget)
+        app_context.close()
+        if self._is_widget:
+            warn_is_widget()
 
     def _execute(self):
+        logger.info("Executing %s", self.name)
         app = None
-        local_scope = {"display": display, "__name__": "__main__", "__file__": str(self.path)}
+        local_scope = {
+            "display": display,
+            "__name__": "__main__",
+            "__file__": str(self.path),
+        }
         ignore = list(local_scope)
         routes: Optional[List[solara.Route]] = None
         if self.path.is_dir():
@@ -76,6 +107,7 @@ class AppScript:
             # resolve the directory, because Path("file").parent.parent == "." != ".."
             self.directory = self.path.resolve()
             routes = solara.generate_routes_directory(self.path)
+            app = solara.autorouting.RenderPage()
         elif self.name.endswith(".py"):
             self.type = AppType.SCRIPT
             # manually add the script to the watcher
@@ -166,6 +198,8 @@ class AppScript:
         if app is None:
             # workaround for backward compatibility
             app = local_scope.get("app")
+        if app is None:
+            app = local_scope.get("page")
         if app is None:
             import difflib
 
@@ -239,13 +273,22 @@ class AppScript:
                     with context:
                         # we save the state for when the app reruns, so we stay in the same state.
                         # (e.g. button clicks, chosen options etc)
-                        context.state = render_context.state_get()
+                        # for instance a dataframe, needs to be pickled, because after the pandas
+                        # module is reloaded, it's a different class type
+                        logger.info("pickling state: %s", render_context.state_get())
+                        try:
+                            context.state = pickle.dumps(render_context.state_get())
+                        except Exception as e:
+                            logger.warning("Could not pickle state, next render the state will be lost: %s", e)
                         # clear/cleanup the render_context, so during reload we start
                         # from scratch
                         context.app_object = None
                         # we want to reuse the container
                         render_context.container = None
-                        render_context.close()
+                        try:
+                            render_context.close()
+                        except Exception as e:
+                            logger.exception("Could not close render context: %s", e)
 
             # ask all contexts/users to reload
             for context in context_values:
@@ -253,29 +296,24 @@ class AppScript:
                     context.reload()
 
 
-# the default app (used in solara-server)
-if "SOLARA_APP" in os.environ:
-    apps["__default__"] = AppScript(os.environ.get("SOLARA_APP", "solara.website.pages:Page"))
-
-
 @dataclasses.dataclass
 class AppContext:
     id: str
     kernel: kernel.Kernel
-    control_sockets: List[WebSocket]
+    control_sockets: List[WebSocket] = dataclasses.field(default_factory=list)
     # this is the 'private' version of the normally global ipywidgets.Widgets.widget dict
     # see patch.py
-    widgets: Dict[str, widgets.Widget]
+    widgets: Dict[str, Widget] = dataclasses.field(default_factory=dict)
     # same, for ipyvue templates
     # see patch.py
-    templates: Dict[str, widgets.Widget]
+    templates: Dict[str, Widget] = dataclasses.field(default_factory=dict)
     user_dicts: Dict[str, Dict] = dataclasses.field(default_factory=dict)
     # anything we need to attach to the context
     # e.g. for a react app the render context, so that we can store/restore the state
     app_object: Optional[Any] = None
     reload: Callable = lambda: None
     state: Any = None
-    container: Optional[widgets.DOMWidget] = None
+    container: Optional[DOMWidget] = None
 
     def display(self, *args):
         print(args)  # noqa
@@ -292,7 +330,11 @@ class AppContext:
         with self:
             if self.app_object is not None:
                 if isinstance(self.app_object, reacton.core._RenderContext):
-                    self.app_object.close()
+                    try:
+                        self.app_object.close()
+                    except Exception as e:
+                        logger.exception("Could not close render context: %s", e)
+                        # we want to continue, so we at least close all widgets
             import solara.server.patch
 
             assert isinstance(widgets.Widget.widgets, solara.server.patch.context_dict_widgets), f"Unexpected widget dict type: {type(widgets.Widget.widgets)}"
@@ -331,6 +373,16 @@ contexts: Dict[str, AppContext] = {}
 current_context: Dict[str, Optional[AppContext]] = {}
 
 
+def create_dummy_context():
+    from . import kernel
+
+    app_context = AppContext(
+        id="dummy",
+        kernel=kernel.Kernel(),
+    )
+    return app_context
+
+
 def get_current_thread_key() -> str:
     thread = threading.currentThread()
     return get_thread_key(thread)
@@ -343,10 +395,14 @@ def get_thread_key(thread: threading.Thread) -> str:
 
 def set_context_for_thread(context: AppContext, thread: threading.Thread):
     key = get_thread_key(thread)
-    contexts[key] = context
     current_context[key] = context
 
 
+def has_current_context() -> bool:
+    thread_key = get_current_thread_key()
+    return (thread_key in current_context) and (current_context[thread_key] is not None)
+
+
 def get_current_context() -> AppContext:
     thread_key = get_current_thread_key()
     if thread_key not in current_context:
@@ -363,10 +419,23 @@ def get_current_context() -> AppContext:
     return context
 
 
-def _run_app(app_state, app_script: AppScript, pathname: str, render_context: reacton.core._RenderContext = None):
+def set_current_context(context: Optional[AppContext]):
+    thread_key = get_current_thread_key()
+    current_context[thread_key] = context
+
+
+def _run_app(
+    app_state,
+    app_script: AppScript,
+    pathname: str,
+    render_context: reacton.core._RenderContext = None,
+):
 
     # app.signal_hook_install()
     main_object = app_script.run()
+    app_state = pickle.loads(app_state) if app_state is not None else None
+    if app_state:
+        logger.info("Restoring state: %r", app_state)
 
     context = get_current_context()
     container = context.container
@@ -414,30 +483,25 @@ def load_app_widget(app_state, app_script: AppScript, pathname: str):
     try:
         render_context = context.app_object
         with context:
-            with reload.reloader.watch():
-                while True:
-                    # reloading might take in extra dependencies, so the reload happens first
-                    if reload.reloader.requires_reload:
-                        reload.reloader.reload()
-                    # reload before starting app, because we may load state using pickle
-                    # if we do that before reloading, the classes are not compatible:
-                    app_state = app_state_initial
-                    # e.g.: _pickle.PicklingError: Can't pickle <class 'testapp.Clicks'>: it's not the same object as testapp.Clicks
-                    try:
-                        widget, render_context = _run_app(app_state, app_script, pathname, render_context=render_context)
-                        if render_context is None:
-                            assert context.container is not None
-                            context.container.children = [widget]
-                    except Exception:
-                        if settings.main.use_pdb:
-                            logger.exception("Exception, will be handled by debugger")
-                            pdb.post_mortem()
-                        raise
+            app_state = app_state_initial
+            try:
+                widget, render_context = _run_app(
+                    app_state,
+                    app_script,
+                    pathname,
+                    render_context=render_context,
+                )
+                if render_context is None:
+                    assert context.container is not None
+                    context.container.children = [widget]
+            except Exception:
+                if settings.main.use_pdb:
+                    logger.exception("Exception, will be handled by debugger")
+                    pdb.post_mortem()
+                raise
 
-                    if render_context:
-                        context.app_object = render_context
-                    if not reload.reloader.requires_reload:
-                        break
+            if render_context:
+                context.app_object = render_context
 
     except BaseException as e:
         error = ""
@@ -510,3 +574,11 @@ def initialize_virtual_kernel(context_id: str, websocket: websocket.WebsocketWra
         kernel.shell_stream = WebsocketStreamWrapper(websocket, "shell")
         kernel.control_stream = WebsocketStreamWrapper(websocket, "control")
         kernel.session.websockets.add(websocket)
+
+
+from . import patch  # noqa
+
+patch.patch()
+# the default app (used in solara-server)
+if "SOLARA_APP" in os.environ:
+    apps["__default__"] = AppScript(os.environ.get("SOLARA_APP", "solara.website.pages:Page"))
