@@ -16,7 +16,7 @@ import solara
 from solara.server.threaded import ServerBase
 
 from . import app as appmod
-from . import cdn_helper, patch, server, websocket
+from . import cdn_helper, server, websocket
 
 os.environ["SERVER_SOFTWARE"] = "solara/" + str(solara.__version__)
 
@@ -48,14 +48,7 @@ class ServerFlask(ServerBase):
     server: Any
 
     def has_started(self):
-        import socket
-
-        s = socket.socket()
-        try:
-            s.connect((self.host, self.port))
-        except ConnectionRefusedError:
-            return False
-        return True
+        return server.is_ready(f"http://{self.host}:{self.port}")
 
     def signal_stop(self):
         assert isinstance(self.server, HTTPServer)
@@ -169,13 +162,16 @@ def read_root(path):
     return response
 
 
+@blueprint.route("/readyz")
+def readyz():
+    json, status = server.readyz()
+    return flask.Response(json, mimetype="application/json", status=status)
+
+
 # using the blueprint and websocket blueprint makes it easier to integrate into other applications
 websocket_extension.init_app(blueprint)
 app = Flask(__name__)
 app.register_blueprint(blueprint)
 
-
-patch.patch()
-
 if __name__ == "__main__":
     app.run(debug=False, port=8765)
