@@ -9,6 +9,7 @@ from uuid import uuid4
 import anyio
 import starlette.websockets
 import uvicorn.server
+import websockets.legacy.http
 from starlette.applications import Starlette
 from starlette.middleware import Middleware
 from starlette.middleware.gzip import GZipMiddleware
@@ -18,11 +19,10 @@ from starlette.routing import Mount, Route, WebSocketRoute
 from starlette.staticfiles import StaticFiles
 
 import solara
-from solara.server import reload
 from solara.server.threaded import ServerBase
 
 from . import app as appmod
-from . import patch, server, settings, telemetry, websocket
+from . import server, settings, telemetry, websocket
 from .cdn_helper import cdn_url_path, default_cache_dir, get_path
 
 os.environ["SERVER_SOFTWARE"] = "solara/" + str(solara.__version__)
@@ -31,6 +31,18 @@ logger = logging.getLogger("solara.server.fastapi")
 # if we add these to the router, the server_test does not run (404's)
 prefix = ""
 
+# The limit for starlette's http traffic should come from h11's DEFAULT_MAX_INCOMPLETE_EVENT_SIZE=16kb
+# In practice, testing with 132kb cookies (server_test.py:test_large_cookie) seems to work fine.
+# For the websocket, the limit is set to 4kb till 10.4, see
+#  * https://github.com/aaugustin/websockets/blob/10.4/src/websockets/legacy/http.py#L14
+# Later releases should set this to 8kb. See
+#  * https://github.com/aaugustin/websockets/commit/8ce4739b7efed3ac78b287da7fb5e537f78e72aa
+#  * https://github.com/aaugustin/websockets/issues/743
+# Since starlette seems to accept really large values for http, lets do the same for websockets
+# An arbitrarily large value we settled on for now is 32kb
+# If we don't do this, users with many cookies will fail to get a websocket connection.
+websockets.legacy.http.MAX_LINE = 1024 * 32
+
 
 class WebsocketWrapper(websocket.WebsocketWrapper):
     ws: starlette.websockets.WebSocket
@@ -214,6 +226,10 @@ class StaticNbFiles(StaticFiles):
 
 
 class StaticPublic(StaticFiles):
+    def lookup_path(self, *args, **kwargs):
+        self.all_directories = self.get_directories(None, None)
+        return super().lookup_path(*args, **kwargs)
+
     def get_directories(
         self,
         directory: Union[str, "os.PathLike[str]", None] = None,
@@ -225,6 +241,10 @@ class StaticPublic(StaticFiles):
 
 
 class StaticAssets(StaticFiles):
+    def lookup_path(self, *args, **kwargs):
+        self.all_directories = self.get_directories(None, None)
+        return super().lookup_path(*args, **kwargs)
+
     def get_directories(
         self,
         directory: Union[str, "os.PathLike[str]", None] = None,
@@ -246,7 +266,6 @@ class StaticCdn(StaticFiles):
 def on_startup():
     # TODO: configure and set max number of threads
     # see https://github.com/encode/starlette/issues/1724
-    reload.reloader.start()
     telemetry.server_start()
 
 
@@ -254,9 +273,15 @@ def on_shutdown():
     telemetry.server_stop()
 
 
+def readyz(request: Request):
+    json, status = server.readyz()
+    return JSONResponse(json, status_code=status)
+
+
 middleware = [Middleware(GZipMiddleware, minimum_size=1000)]
 
 routes = [
+    Route("/readyz", endpoint=readyz),
     Route("/jupyter/api/kernels/{id}", endpoint=kernels),
     WebSocketRoute("/jupyter/api/kernels/{id}/{name}", endpoint=kernel_connection),
     Route("/", endpoint=root),
@@ -278,4 +303,3 @@ app = Starlette(routes=routes, on_startup=[on_startup], on_shutdown=[on_shutdown
 
 # routes_test_sub = [Route("/", endpoint=myroot), Mount("/foo/", routes=routes)]
 # app = Starlette(routes=routes_test_sub, on_startup=[on_startup], on_shutdown=[on_shutdown], middleware=middleware)
-patch.patch()
